import random
import gmpy2
import bisect

def perfect_stats_prng(n_bits):
	x = ''
	for i in range((n_bits)//2):
		bit = random.getrandbits(1)
		x += str(bit) + str(1-bit)
	return int(x, 2)

def generate_prime(n_bits):
	x = 4
	while not gmpy2.is_prime(x):
		x = perfect_stats_prng(n_bits-2)
		x = x*2
		x = 2**(n_bits-1) + x | 1

	return x

print('Computing primes...')
p = generate_prime(1024)
q = generate_prime(1024)
n = p*q
print(n)
print('Primes computed')

KEEP = 1000    # How many pairs to keep

def find_solsk(n, k, solsk_minus_one):
	solsk = []
	seen = {}

	n_mod_k = n % 2**k
	for p,q in solsk_minus_one:
		p2 = p + 2**k
		q2 = q + 2**k
		p =  p + 2**(k+1)
		q =  q + 2**(k+1)

		n1 = (p*q) % (2**(k))
		n2 = (p2*q) % (2**(k))
		n3 = (p*q2) % (2**(k))
		n4 = (p2*q2) % (2**(k))

		x1,y1 = min(p,q), max(p,q)
		x2,y2 = min(q2,p), max(q2,p)
		x3,y3 = min(q,p2), max(q,p2)
		x4,y4 = min(p2,q2), max(p2,q2)

		if n1 == n_mod_k and (x1,y1) not in seen:
			bisect.insort(solsk, ((p,q)))
			seen[(x1,y1)] = True

		if n2 == n_mod_k and (x2,y2) not in seen:
			bisect.insort(solsk, ((p,q2)))
			seen[(x2,y2)] = True

		if n3 == n_mod_k and (x3,y3) not in seen:
			bisect.insort(solsk, ((p2,q)))
			seen[(x3,y3)] = True

		if n4 == n_mod_k and (x4,y4) not in seen:
			bisect.insort(solsk, ((p2,q2)))
			seen[(x4,y4)] = True

	return solsk[:KEEP] # Discard everything but the KEEP best possibilities


sols = [(1,1)] #We know that for both primes, the lsb is 1

for k in range(1, 1022, 2):
	print(k,'/',1024,len(sols))
	sols = find_solsk(n, k, sols)

for p,q in sols:
	p2 = p + 2**1023
	q2 = q + 2**1023
	if n % p2 == 0 or n % q2 == 0:
		print('Found it:', p2)
		break